#!/usr/bin/env python3
# -*- coding: utf-8 -*-
############################################################################
#             #                                                            #
#   #####     # (!) Giovanni Squillero <giovanni.squillero@polito.it>      #
#  ######     #                                                            #
#  ###   \    # Copying and distributing this file, either with or without #
#   ##G  c\   # modification, is permitted in any medium without royalty,  #
#   #     _\  # provided that this 10-line comment is preserved.           #
#   |  _/     # THIS FILE IS OFFERED AS-IS, WITHOUT ANY WARRANTY.          #
#             #                                                            #
############################################################################

import sys
import logging

assert sys.version_info >= (3, 4)

class SectionContratints:
    check_list = None

    def __init__(self):
        self.check_list = list()

    def check(self, real_section):
        return all([f(real_section) for f in self.check_list])


class SectionIndividual:
    nodes = None
    def __init__(self):
        self.nodes = list()


def check_min_number_of_nodes(min):
    """ Long live closures! """
    logging.info("New min: %d", min)
    def checker(section):
        return len(section.nodes) >= min
    return checker


def check_max_number_of_nodes(max):
    """ Long live closures! """
    logging.info("New max: %d", max)
    def checker(section):
        return len(section.nodes) <= max
    return checker


if __name__ == "__main__":
    logging.basicConfig(format="%(asctime)s %(levelname)s:: %(message)s", datefmt="%H:%M:%S")
    logging.getLogger().setLevel(level=logging.INFO)

    sez1 = SectionIndividual()
    sez1.nodes = [23, 10, 19, 70]
    sez2 = SectionIndividual()
    sez2.nodes = [42]   

    zap = SectionContratints()

    zap.check_list.append(check_max_number_of_nodes(10))
    logging.info(zap.check(sez1))
    logging.info(zap.check(sez2))

    logging.info("")

    zap.check_list.append(check_min_number_of_nodes(2))
    logging.info(zap.check(sez1))
    logging.info(zap.check(sez2))

    logging.info("")

    _ = check_min_number_of_nodes(999)
    logging.info(zap.check(sez1))
    logging.info(zap.check(sez2))

    # Play with list of classes
    logging.info("")
    x = [SectionIndividual, SectionContratints]
    logging.info(x)
    zap = x[0]()
    logging.info(type(zap))
