from random import randint

from microgp4.ctg.parameters.var_type import VarType


class BitVector(VarType):
    """Bit vector of 8 as max length"""
    __max_length = 8
    __max_value = (2 ** __max_length) - 1

    def mutate(self, strength):
        m = '0' * self.__max_length
        m_l = list(m)
        iterations = int(float(self.__max_length) * strength)
        for i in range(iterations):
            index = randint(0, self.__max_length - 1)
            m_l[index] = '1'
        m = ''.join(m_l)
        self._value ^= int(m, 2)

    @staticmethod
    def generate_new_value(node_id):
        return randint(0, BitVector.__max_value)

    def __str__(self):
        return bin(self.value)[2:].zfill(self.__max_length)

    @staticmethod
    def check_value(node_id, value):
        return isinstance(value, int) and value < BitVector.__max_value


def class_bit_vector(max_length, format_='{}'):
    assert isinstance(max_length, int) and max_length > 0

    class BinVectorI(BitVector):
        __max_length = max_length
        __max_value = (2 ** __max_length) - 1

        @staticmethod
        def generate_new_value(node_id):
            return randint(0, BinVectorI.__max_value)

        @staticmethod
        def check_value(node_id, value):
            return isinstance(value, int) and value < BinVectorI.__max_value

        def __str__(self):
            return format_.format(self.value)

    return BinVectorI
