from abc import abstractmethod
from random import randint, uniform

from microgp4.ctg.parameters.var_type import VarType


class EnumType(VarType):
    @abstractmethod
    def __str__(self):
        pass

    @staticmethod
    @abstractmethod
    def check_value(node_id, value):
        pass

    @abstractmethod
    def mutate(self, strength):
        pass

    @staticmethod
    @abstractmethod
    def generate_new_value(node_id):
        pass


def class_enum(value_tuple, format_='{}'):
    assert isinstance(value_tuple, tuple)
    max_len = len(value_tuple) - 1
    assert max_len >= 0

    class _EnumType(EnumType):
        def mutate(self, strength):
            i = value_tuple.index(self.value)
            s = 1
            if strength > 0.5:
                s = -1
            i += int(float(strength) * float(uniform(0, max_len)) * s)
            i = max(min(i, max_len), 0)
            return value_tuple[i]

        @staticmethod
        def generate_new_value(node_id):
            return value_tuple[randint(0, max_len)]

        def __str__(self):
            return format_.format(self.value)

        @staticmethod
        def check_value(node_id, value):
            return value in value_tuple

    return _EnumType
