from random import random, uniform

from microgp4.ctg.parameters.var_type import VarType


class FloatType(VarType):
    """Float between 0 and 1"""

    def mutate(self, strength):
        s = -1
        if strength > 0.5:
            s = 1
        self._value += FloatType.generate_new_value(self._node_id) * strength * s
        max(min(self.value, 1), 0)

    @staticmethod
    def generate_new_value(node_id):
        return random()

    def __str__(self):
        return str(self._value)

    @staticmethod
    def check_value(node_id, value):
        return isinstance(value, float) and 0 <= value <= 1


def class_float_range(min_value, max_value, format_='{}'):
    assert isinstance(max_value, float) or isinstance(max_value, int)
    assert isinstance(min_value, float) or isinstance(min_value, int)
    assert min_value < max_value

    class __FloatRange(FloatType):
        @staticmethod
        def check_value(node_id, value):
            return isinstance(value, float) and min_value <= value <= max_value

        def mutate(self, strength):
            s = -1
            if strength > 0.5:
                s = 1
            self._value += self.generate_new_value(self._node_id) * strength * s
            max(min(self.value, max_value), min_value)

        @staticmethod
        def generate_new_value(node_id):
            return uniform(min_value, max_value)

        def __str__(self):
            return format_.format(self.value)

    return __FloatRange
