from abc import abstractmethod
from random import randint, random

from microgp4.ctg.parameters.var_type import VarType


class IntegerType(VarType):
    @staticmethod
    def check_value(node_id, value):
        return isinstance(value, int)

    def __str__(self):
        return str(self.value)

    @staticmethod
    @abstractmethod
    def generate_new_value(node_id):
        pass


def class_inter_range(min_value, max_value, format_='{}'):
    assert IntegerType.check_value(None, min_value)
    assert IntegerType.check_value(None, max_value)
    assert min_value < max_value

    class _IntegerRange(IntegerType):
        @staticmethod
        def check_value(node_id, value):
            if IntegerType.check_value(node_id, value):
                return min_value <= value <= max_value
            else:
                return False

        def mutate(self, strength):
            r = random()
            s = -1
            if r > 0.5:
                s = 1
            self._value += (int(float(strength) * float(_IntegerRange.generate_new_value(self._node_id))) * s)
            self._value = max(min(self._value, max_value), min_value)

        @staticmethod
        def generate_new_value(node_id):
            return randint(min_value, max_value)

        def __str__(self):
            return format_.format(self.value)

    return _IntegerRange
