#!/bin/bash


python_test_path="./python/example_01.py"

base_working_directory="/tmp/microgp4_$$_example_01"
result_directory="./result_test_01"

if [[ ! -e ${base_working_directory} ]] ; then
    mkdir ${base_working_directory}
fi

if [[ ! -e ${result_directory} ]] ; then
    mkdir ${result_directory}
fi

json_file_path="./json/example_01/sections.json"
assembly_path=${base_working_directory}"/result.s"

main_c_path="./c/main.c"
executable_path=${base_working_directory}"/test_01"

BEST_COLLISION_RESULT=10000

if [[ $# -gt 0 ]] ; then
    iteration_number=$1
else
   iteration_number=100 
fi

total_iteration=${iteration_number}

echo "JSON file location: ${json_file_path}"

export PYTHONPATH="$PYTHONPATH:../main/python/"

global_start_time="$(date +%s.%N)"

while [[ ${iteration_number} -gt 0 ]] ; do

    start_time="$(date -u +%s.%N)"

    ((iteration_number-=1))

    python3 ${python_test_path} -i ${json_file_path} -o ${assembly_path}

    gcc -o ${executable_path} ${main_c_path} ${assembly_path}

    res=$(${executable_path})

    collision_text="Number of collision: ${res}"

    if [[ ${BEST_COLLISION_RESULT} -gt ${res} ]] ; then
        BEST_COLLISION_RESULT=${res}
        collision_text="${collision_text} | Found new best program! Saved!"
        cp ${assembly_path} ${result_directory}"/${res}_result.s"
    else
        collision_text="${collision_text} | Skipping result               "
    fi

    end_time="$(date -u +%s.%N)"

    elapsed=$(echo "((${end_time}-${start_time}) * ${iteration_number})" | bc | awk '{printf "%.2f", $0}')
    percentage=$(echo "100 * (${total_iteration} - ${iteration_number}) / ${total_iteration}" | bc | awk '{printf "%.2f", $0}')

    echo -ne "${collision_text} | Best result: ${BEST_COLLISION_RESULT} | Time left: ${elapsed}s | ${percentage}% \033[0K\r"

done

end_time="$(date +%s.%N)"
total_time=$(echo "${end_time} - ${global_start_time}" | bc | awk '{printf "%.2f", $0}')

echo -ne '\n'

echo "Best collision result: ${BEST_COLLISION_RESULT} | Operation end in ${total_time}s"
